import type { AdListProps } from '../types/adList';
import type { AdColumn, AdFetchParams } from '/@/components/AdTable';
import type { AdDetailFormProps } from '/@/components/AdTable/src/types/adDetailForm';
import { FormSchema } from '/@/components/Table';
import { TagEditor } from '/@/components/Tag';
import { unref, computed, h } from 'vue';
import type { ComputedRef } from 'vue';
// import type { ComponentType } from '/@/components/Form/src/types';
import { useI18n } from '/@/hooks/web/useI18n';
import { isBoolean, isString, isArray, isFunction } from '/@/utils/is';
import md5 from 'crypto-js/md5';

import { getInputNumberPrecision } from '/@/components/AdTable/src/hooks/index';

import {
  isEmail,
  isIpv4Address,
  isIpv6Address,
  isUrl,
  isMobileNumber,
  isIdCard,
  isGuid,
} from '/@/enums/regexEnum';

/**
 * 使用高级详情表单
 * @param propsRef 高级表格选项
 * @param reloadFunc 重新加载方法
 * @returns 高级详情表单对象
 */
export function useAdDetailForm(
  propsRef: ComputedRef<AdListProps>,
  reloadFunc?: (opt?: AdFetchParams) => Promise<void>,
) {
  const { t } = useI18n();

  /** 获取详情窗体属性 */
  const getDetailFormProps = computed((): Partial<AdDetailFormProps> => {
    const { columns, detailForm } = unref(propsRef);
    const formSchemaColumns = renderAdDetailFormSchema(columns, detailForm);
    const formSchema = getFormSchema(formSchemaColumns);

    // 获取并处理表单标识
    let formIndex = detailForm?.formIndex;
    if (formIndex === '' || formIndex === null || formIndex === undefined) {
      const formSchemaObj: object[] = [];
      if (formSchema && formSchema.length > 0) {
        formSchema.forEach((item) => {
          formSchemaObj.push({ field: item.field, component: item.component });
        });
      }
      formIndex = md5(JSON.stringify(formSchemaObj)).toString();
      // console.log('formIndex', formIndex);
      // console.log('formIndex1', formSchemaObj);
      // console.log('formIndex2', JSON.stringify(formSchemaObj));
      // console.log('formIndex3', md5(JSON.stringify(formSchemaObj)));
    }

    let formHelpMessage = detailForm?.helpMessage;
    if (detailForm?.showFormSchemaInfo !== false) {
      const formSchemaInfo = `${t('common.form')}：${formIndex}`;
      if (!formHelpMessage) {
        formHelpMessage = formSchemaInfo;
      } else if (isString(formHelpMessage)) {
        formHelpMessage = [formHelpMessage, formSchemaInfo];
      } else if (isArray(formHelpMessage)) {
        formHelpMessage = [...formHelpMessage, formSchemaInfo];
      }
    }

    return {
      /** 表单标识 */
      formIndex: formIndex,
      /** 表单抬头	-	modal 标题 */
      title: detailForm?.title,
      /** 固定 modal 的宽度 */
      width: detailForm?.width || '50%',
      /** 固定 modal 的高度 */
      height: detailForm?.height,
      /** modal 的最小高度 */
      minHeight: detailForm?.minHeight,
      /** 是否开启拖拽 */
      draggable: detailForm?.draggable,
      /** 是否开启自适应高度，开启后会跟随屏幕变化自适应内容，并出现滚动条 */
      useWrapper: detailForm?.useWrapper,
      /** 开启是适应高度后，如果超过屏幕高度，底部和顶部会保持一样的间距，该参数可以用来缩小底部的间距 */
      wrapperFooterOffset: detailForm?.wrapperFooterOffset,
      /** 是否可以进行全屏 */
      canFullscreen: detailForm?.canFullscreen,
      /** 默认全屏 */
      defaultFullscreen: detailForm?.defaultFullscreen,
      /** loading 状态 */
      loading: detailForm?.loading,
      /** loading 文本 */
      loadingTip: detailForm?.loadingTip,
      /** 显示关闭按钮 */
      showCancelBtn: detailForm?.showCancelBtn,
      /** 显示确认按钮 */
      showOkBtn: detailForm?.showOkBtn,
      /** 标题右侧提示文本 */
      helpMessage: formHelpMessage,
      /** 是否居中弹窗 */
      centered: detailForm?.centered,
      /** 关闭按钮文本 */
      cancelText: detailForm?.cancelText,
      /** 确认按钮文本 */
      okText: detailForm?.okText,
      /** 表单架构 */
      formSchema: detailForm?.formSchema || formSchema,
      /** 扩展 form 组件，增加 label 宽度，表单内所有组件适用，可以单独在某个项覆盖或者禁用 */
      labelWidth: detailForm?.labelWidth || 139,
      /** label 布局 */
      labelAlign: detailForm?.labelAlign,
      /** 紧凑类型表单，减少 margin-bottom */
      compact: detailForm?.compact,
      /** 向表单内所有组件传递 size 参数,自定义组件需自行实现 size 接收 */
      size: detailForm?.size,
      /** 向表单内所有组件传递 disabled 属性，自定义组件需自行实现 disabled 接收 */
      disabled: detailForm?.disabled,
      /** 实体新增接口请求对象 */
      createApi: detailForm?.createApi,
      /** 实体更新接口请求对象 */
      updateApi: detailForm?.updateApi,
      /** 实体删除接口请求对象 */
      deleteApi: detailForm?.deleteApi,
      /** 关闭回调，返回true关闭窗体 */
      closeFunc: detailForm?.closeFunc,
      /** 提交回调，返回true关闭窗体 */
      submitFunc: detailForm?.submitFunc,
      /** 重新加载表格数据方法 */
      reloadFunc: detailForm?.reloadFunc || reloadFunc,
      // reloadFunc: detailForm?.reloadFunc ? detailForm?.reloadFunc : reloadFunc,
    };
  });

  //#region 渲染高级详情表单列表单架构
  /**
   * 渲染高级详情表单列表单架构
   * @param columns 列集合
   * @returns 处理后的高级列集合
   */
  function renderAdDetailFormSchema(
    columns: AdColumn[],
    detailForm?: AdDetailFormProps,
  ): AdColumn[] {
    const data: AdColumn[] = [];

    columns.forEach((item) => {
      const newItem = { ...item };
      //渲染表单架构
      const componentType = newItem.formSchema?.component || newItem.editComponent || undefined;
      newItem.formSchema = {
        field: newItem.dataIndex as string,
        label: newItem.title as string,
        component: componentType || 'Input',
        ...newItem.formSchema,
      };
      if (newItem.formVisible === false) {
        newItem.formSchema = {
          ...newItem.formSchema,
          ifShow: false,
          show: false,
        };
        // 加入至新集合
        data.push(newItem);
      } else {
        if (isBoolean(newItem.required) && !isFunction(newItem.formSchema.required)) {
          newItem.formSchema.required = newItem.formSchema.required || newItem.required;
        }
        if (isBoolean(newItem.formEditable) && !isFunction(newItem.formSchema.dynamicDisabled)) {
          newItem.formSchema.dynamicDisabled = !newItem.formEditable;
        }
        if (newItem.editComponentProps && !isFunction(newItem.formSchema.componentProps)) {
          newItem.formSchema.componentProps = {
            ...newItem.formSchema.componentProps,
            ...newItem.editComponentProps,
          };
        }
        const formCustomer = newItem.formCustomer;
        //初始化编辑控件
        switch (newItem.dataType) {
          case 'enum':
            newItem.editComponent = newItem.editComponent || 'Select';
            newItem.formSchema.component = componentType || newItem.editComponent;
            if (
              newItem.formSchema.component === 'Select' &&
              newItem.enumData &&
              newItem.enumData.length > 0 &&
              !isFunction(newItem.formSchema.componentProps)
            ) {
              // @ts-ignore
              const showSearch = newItem.formSchema.componentProps?.showSearch;
              // @ts-ignore
              const optionFilterProp = newItem.formSchema.componentProps?.optionFilterProp;
              newItem.formSchema.componentProps = {
                ...newItem.formSchema.componentProps,
                options: [...newItem.enumData],
                showSearch: isBoolean(showSearch) ? showSearch : true,
                optionFilterProp: optionFilterProp || 'label',
              };
            }
            break;
          case 'integer':
          case 'number':
          case 'quantity':
          case 'price':
          case 'amount':
          case 'rate':
          case 'percentage':
          case 'measure':
            newItem.editComponent = newItem.editComponent || 'InputNumber';
            newItem.formSchema.component = componentType || newItem.editComponent;
            if (
              newItem.formSchema.component === 'InputNumber' &&
              !isFunction(newItem.formSchema.componentProps)
            ) {
              const numberPrecision = getInputNumberPrecision(newItem.dataType);
              newItem.formSchema.componentProps = {
                ...newItem.formSchema.componentProps,
                precision: numberPrecision >= 0 ? numberPrecision : undefined,
              };
            }
            break;
          case 'string':
          case 'email':
          case 'mobile':
          case 'url':
          case 'ipv4':
          case 'ipv6':
          case 'idcard':
          case 'guid':
            newItem.editComponent = newItem.editComponent || 'Input';
            newItem.formSchema.component = componentType || newItem.editComponent;
            if (newItem.formSchema.component === 'Input') {
              if (newItem.formSchema.rules === undefined || newItem.formSchema.rules === null) {
                newItem.formSchema.rules = [];
              }
              newItem.formSchema.rules.push({
                required: newItem.required,
                // @ts-ignore
                validator: async (rule, value) => {
                  if (value) {
                    let isFormatOk = true;
                    switch (newItem.dataType) {
                      case 'email':
                        isFormatOk = isEmail(value);
                        break;
                      case 'mobile':
                        isFormatOk = isMobileNumber(value);
                        break;
                      case 'url':
                        isFormatOk = isUrl(value);
                        break;
                      case 'ipv4':
                        isFormatOk = isIpv4Address(value);
                        break;
                      case 'ipv6':
                        isFormatOk = isIpv6Address(value);
                        break;
                      case 'idcard':
                        isFormatOk = isIdCard(value);
                        break;
                      case 'guid':
                        isFormatOk = isGuid(value);
                        break;
                    }
                    if (!isFormatOk) {
                      return Promise.reject(`${newItem.dataType}${t('common.formatErrorText')}`);
                    }
                    return Promise.resolve();
                  }
                  if (!value && newItem.required) {
                    return Promise.reject();
                  }
                  return Promise.resolve();
                },
                trigger: ['change', 'blur'],
              });
            }
            break;
          case 'text':
            // @ts-ignore
            newItem.editComponent = newItem.editComponent || 'InputTextArea';
            // @ts-ignore
            newItem.formSchema.component = componentType || newItem.editComponent;
            if (
              newItem.formSchema.component === 'InputTextArea' &&
              !isFunction(newItem.formSchema.componentProps)
            ) {
              // @ts-ignore
              const showCount = newItem.formSchema.componentProps?.showCount;
              newItem.formSchema.componentProps = {
                ...newItem.formSchema.componentProps,
                showSearch: isBoolean(showCount) ? showCount : true,
                // autosize: { minRows: 2, maxRows: 6 },
              };
            }
            break;
          case 'password':
            // @ts-ignore
            newItem.editComponent = newItem.editComponent || 'InputPassword';
            // @ts-ignore
            newItem.formSchema.component = componentType || newItem.editComponent;
            break;
          case 'boolean':
            newItem.editComponent = newItem.editComponent || 'Switch';
            newItem.formSchema.component = componentType || newItem.editComponent;
            if (
              newItem.formSchema.component === 'Switch' ||
              newItem.formSchema.component === 'Checkbox'
            ) {
              const defaultValue = newItem.formSchema.defaultValue;
              newItem.formSchema.defaultValue = isBoolean(defaultValue) ? defaultValue : false;
            }
            break;
          case 'datetime':
          case 'date':
            newItem.editComponent = newItem.editComponent || 'DatePicker';
            newItem.formSchema.component = componentType || newItem.editComponent;
            if (
              newItem.formSchema.component === 'DatePicker' &&
              !isFunction(newItem.formSchema.componentProps)
            ) {
              // @ts-ignore
              let showTime = newItem.formSchema.componentProps?.showTime;
              showTime = isBoolean(showTime) ? showTime : newItem.dataType === 'datetime';
              newItem.formSchema.componentProps = {
                ...newItem.formSchema.componentProps,
                showTime: showTime,
                valueFormat: showTime ? 'YYYY-MM-DD HH:mm:ss' : 'YYYY-MM-DD',
              };
            }
            break;
          case 'time':
            newItem.editComponent = newItem.editComponent || 'TimePicker';
            newItem.formSchema.component = componentType || newItem.editComponent;
            break;
          case 'array':
            const formSchemaRender = ({ model, field }) => {
              return h(TagEditor, {
                value: model[field],
                editEnabled: newItem.formEditable,
                addTagText: t('common.addText'),
                onChange: (tags: string[]) => {
                  model[field] = tags;
                },
              });
            };
            newItem.formSchema = {
              ...newItem.formSchema,
              colProps: {
                ...newItem.formSchema?.colProps,
                ...formCustomer?.colProps,
                // span: newItem.formSchema?.colProps?.span || formCustomer?.colProps?.span || 24,
                xs: newItem.formSchema?.colProps?.xs || formCustomer?.colProps?.xs || 24, // <576px
                sm: newItem.formSchema?.colProps?.sm || formCustomer?.colProps?.sm || 24, // ≥576px
                md: newItem.formSchema?.colProps?.md || formCustomer?.colProps?.md || 24, // ≥768px
                lg: newItem.formSchema?.colProps?.lg || formCustomer?.colProps?.lg || 24, // ≥992px
                xl: newItem.formSchema?.colProps?.xl || formCustomer?.colProps?.xl || 24, // ≥1200px
                xxl: newItem.formSchema?.colProps?.xxl || formCustomer?.colProps?.xxl || 24, // ≥1600px
                // xxxl: newItem.formSchema?.colProps?.xxxl || formCustomer?.colProps?.xxxl || 24, // ≥2000px 3.0
              },
              render: newItem.formSchema?.render || formCustomer?.render || formSchemaRender,
            };
            break;
        }

        // 根据控件类型设置布局
        switch (newItem.formSchema.component) {
          case 'Select':
          case 'ApiSelect':
          case 'ApiTreeSelect':
          case 'InputNumber':
          case 'Input':
          case 'InputPassword':
          case 'DatePicker':
          case 'TimePicker':
          case 'InputSearch':
          case 'InputCountDown':
          case 'TreeSelect':
          case 'MonthPicker':
          case 'RangePicker':
          case 'WeekPicker':
          case 'StrengthMeter':
          case 'IconPicker':
          case 'AutoComplete': // 自动完成
          case 'Cascader': // 级联选择
          case 'ApiCascader': // 级联选择
          case 'Rate': // 评分
          case 'Slider': // 滑动输入条
            if (!isFunction(newItem.formSchema.componentProps)) {
              switch (newItem.formSchema.component) {
                case 'Select':
                case 'ApiSelect':
                case 'ApiTreeSelect':
                  // @ts-ignore
                  const showSearch = newItem.formSchema.componentProps?.showSearch;
                  // @ts-ignore
                  const optionFilterProp = newItem.formSchema.componentProps?.optionFilterProp;
                  newItem.formSchema.componentProps = {
                    ...newItem.formSchema.componentProps,
                    showSearch: isBoolean(showSearch) ? showSearch : true,
                    optionFilterProp: optionFilterProp || newItem.dataIndex || 'lable',
                  };
                  break;
                case 'InputNumber':
                  let dataType:
                    | 'integer'
                    | 'number'
                    | 'quantity'
                    | 'price'
                    | 'amount'
                    | 'rate'
                    | 'percentage'
                    | 'measure' = 'number';
                  switch (newItem.dataType) {
                    case 'integer':
                    case 'number':
                    case 'quantity':
                    case 'price':
                    case 'amount':
                    case 'rate':
                    case 'percentage':
                    case 'measure':
                      dataType = newItem.dataType;
                      break;
                  }
                  const numberPrecision = getInputNumberPrecision(dataType);
                  newItem.formSchema.componentProps = {
                    ...newItem.formSchema.componentProps,
                    precision: numberPrecision >= 0 ? numberPrecision : undefined,
                  };
                  break;
              }

              newItem.formSchema.componentProps = {
                ...newItem.formSchema.componentProps,
                style: {
                  // @ts-ignore
                  ...newItem.formSchema.componentProps?.style,
                  // @ts-ignore
                  width: newItem.formSchema.componentProps?.style?.width || '100%',
                },
              };
            }

            newItem.formSchema.colProps = {
              ...newItem.formSchema.colProps,
              ...formCustomer?.colProps,
              // span: newItem.formSchema?.colProps?.span || formCustomer?.colProps?.span || 12,
              xs: newItem.formSchema?.colProps?.xs || formCustomer?.colProps?.xs || 24, // <576px
              sm: newItem.formSchema?.colProps?.sm || formCustomer?.colProps?.sm || 24, // ≥576px
              md: newItem.formSchema?.colProps?.md || formCustomer?.colProps?.md || 24, // ≥768px
              lg: newItem.formSchema?.colProps?.lg || formCustomer?.colProps?.lg || 24, // ≥992px
              xl: newItem.formSchema?.colProps?.xl || formCustomer?.colProps?.xl || 12, // ≥1200px
              xxl: newItem.formSchema?.colProps?.xxl || formCustomer?.colProps?.xxl || 12, // ≥1600px
              // xxxl: newItem.formSchema?.colProps?.xxxl || formCustomer?.colProps?.xxxl || 12, // ≥2000px 3.0
            };
            break;
          case 'InputTextArea':
          case 'InputGroup':
          case 'ApiTree':
          case 'ApiRadioGroup':
          case 'RadioButtonGroup':
          case 'RadioGroup':
          case 'CheckboxGroup':
            newItem.formSchema.componentProps = {
              ...newItem.formSchema.componentProps,
              style: {
                // @ts-ignore
                ...newItem.formSchema.componentProps?.style,
                // @ts-ignore
                width: newItem.formSchema.componentProps?.style?.width || '100%',
              },
            };
            newItem.formSchema.colProps = {
              ...newItem.formSchema.colProps,
              ...formCustomer?.colProps,
              // span: newItem.formSchema?.colProps?.span || formCustomer?.colProps?.span || 12,
              xs: newItem.formSchema?.colProps?.xs || formCustomer?.colProps?.xs || 24, // <576px
              sm: newItem.formSchema?.colProps?.sm || formCustomer?.colProps?.sm || 24, // ≥576px
              md: newItem.formSchema?.colProps?.md || formCustomer?.colProps?.md || 24, // ≥768px
              lg: newItem.formSchema?.colProps?.lg || formCustomer?.colProps?.lg || 24, // ≥992px
              xl: newItem.formSchema?.colProps?.xl || formCustomer?.colProps?.xl || 24, // ≥1200px
              xxl: newItem.formSchema?.colProps?.xxl || formCustomer?.colProps?.xxl || 12, // ≥1600px
              // xxxl: newItem.formSchema?.colProps?.xxxl || formCustomer?.colProps?.xxxl || 12, // ≥2000px 3.0
            };
            break;
          case 'Switch':
          case 'Checkbox':
            newItem.formSchema.colProps = {
              ...newItem.formSchema.colProps,
              ...formCustomer?.colProps,
              // span: newItem.formSchema?.colProps?.span || formCustomer?.colProps?.span || 12,
              xs: newItem.formSchema?.colProps?.xs || formCustomer?.colProps?.xs || 24, // <576px
              sm: newItem.formSchema?.colProps?.sm || formCustomer?.colProps?.sm || 24, // ≥576px
              md: newItem.formSchema?.colProps?.md || formCustomer?.colProps?.md || 12, // ≥768px
              lg: newItem.formSchema?.colProps?.lg || formCustomer?.colProps?.lg || 12, // ≥992px
              xl: newItem.formSchema?.colProps?.xl || formCustomer?.colProps?.xl || 8, // ≥1200px
              xxl: newItem.formSchema?.colProps?.xxl || formCustomer?.colProps?.xxl || 6, // ≥1600px
              // xxxl: newItem.formSchema?.colProps?.xxxl || formCustomer?.colProps?.xxxl || 12, // ≥2000px 3.0
            };
            break;
          case 'Upload':
          case 'Render':
          case 'Divider':
            break;
        }
        // 处理渲染呈现属性
        if (!newItem.formSchema.render && formCustomer?.render) {
          newItem.formSchema.render = formCustomer.render;
        }
        // 将未设置的属性赋默认值
        if (!newItem.formSchema.field) {
          newItem.formSchema.field = newItem.dataIndex as string;
        }
        if (!newItem.formSchema.label) {
          newItem.formSchema.label = newItem.title as string;
        }
        if (detailForm?.showFormSchemaInfo !== false) {
          const formSchemaInfo = `${t('common.field')}：${newItem.dataIndex}`;
          if (!newItem.formSchema.helpMessage) {
            newItem.formSchema.helpMessage = formSchemaInfo;
          } else if (isString(newItem.formSchema.helpMessage)) {
            newItem.formSchema.helpMessage = [newItem.formSchema.helpMessage, formSchemaInfo];
          } else if (isArray(newItem.formSchema.helpMessage)) {
            newItem.formSchema.helpMessage = [...newItem.formSchema.helpMessage, formSchemaInfo];
          }
        }

        // 加入至新集合
        data.push(newItem);
      }
    });
    return data;
  }
  //#endregion

  /** 获取详情表单架构 */
  function getFormSchema(columns: AdColumn[]) {
    const formSchemaData: FormSchema[] = [];
    if (columns && columns.length > 0) {
      columns.forEach((item) => {
        if (item.formSchema) {
          formSchemaData.push(item.formSchema);
        }
      });
    }

    return formSchemaData;
  }

  return { getDetailFormProps };
}
