/** 正则枚举 */
export enum RegexEnum {
  /** IPV4的匹配字符串 */
  IPV4 = '(([1-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){1}' + // initial first field, 1-255
    '(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){2}' + // following 2 fields, 0-255 followed by .
    '([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])', // final field, 0-255
  /** IPV6的匹配字符串 */
  IPV6 = 's*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]d|1dd|[1-9]?d)(.(25[0-5]|2[0-4]d|1dd|[1-9]?d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]d|1dd|[1-9]?d)(.(25[0-5]|2[0-4]d|1dd|[1-9]?d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]d|1dd|[1-9]?d)(.(25[0-5]|2[0-4]d|1dd|[1-9]?d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]d|1dd|[1-9]?d)(.(25[0-5]|2[0-4]d|1dd|[1-9]?d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]d|1dd|[1-9]?d)(.(25[0-5]|2[0-4]d|1dd|[1-9]?d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]d|1dd|[1-9]?d)(.(25[0-5]|2[0-4]d|1dd|[1-9]?d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]d|1dd|[1-9]?d)(.(25[0-5]|2[0-4]d|1dd|[1-9]?d)){3}))|:)))(%.+)?s*',
  /** 前后断言的字符串截取匹配字符串 */
  SUBSTRING_FORMAT = '(?<=({0})).+(?=({1}))',
  /** 邮箱的匹配字符串 */
  EMAIL = '[a-zA-Z0-9_]+@[a-zA-Z0-9]+(\\.[a-zA-Z]+)+',
  /** Unicode（中文）字符的匹配字符串 */
  UNICODE = '[\u4E00-\u9FA5\uE815-\uFA29]+',
  /** URL的匹配字符串 */
  URL = '(http|https|ftp|rtsp|mms):(//|\\\\)[A-Za-z0-9%-_@]+.[A-Za-z0-9%-_@]+[A-Za-z0-9./=?%-&_~`@:+!;]*',
  /** 手机号的匹配字符串 */
  MOBILE_NUMBER = '((13[0-9])|(14[5,7,9])|(15[^4])|(18[0-9])|(17[0,1,3,5,6,7,8]))[0-9]{8}',
  /** 整数的匹配字符串 */
  INTEGER = '-?[0-9]+',
  /** 身份证的匹配字符串 */
  ID_CARD = '[1-9]d{7}((0d)|(1[0-2]))(([0|1|2]d)|3[0-1])d{3}$|^[1-9]d{5}[1-9]d{3}((0d)|(1[0-2]))(([0|1|2]d)|3[0-1])d{3}([0-9]|X)',
  /** GUID的匹配字符串 */
  GUID = '[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}',
}

/**
 * 正则表达式模式字符串
 * @param regexEnum 正则枚举
 */
export function patternStr(regexEnum: RegexEnum): string {
  if (regexEnum === undefined || regexEnum === null) {
    return `^$`;
  }
  const patternString = `^${regexEnum}$`;
  return patternString;
}

/**
 * 正则表达式模式
 * @param regexEnum 正则枚举
 */
export function pattern(regexEnum: RegexEnum): RegExp {
  const patternString = patternStr(regexEnum);
  return new RegExp(patternString);
}

/**
 * 所指定的正则表达式在指定的输入字符串中是否找到了匹配项
 * @param regExp 要匹配的正则表达式模式
 * @param value 要搜索匹配项的字符串
 */
export function isMatch(regExp: RegExp, value: string): boolean {
  if (regExp) {
    return regExp.test(value);
  }
  return false;
}

/**
 * 是否电子邮件
 * @param value 要搜索匹配项的字符串
 */
export function isEmail(value: string): boolean {
  const emailPattern = pattern(RegexEnum.EMAIL);
  return isMatch(emailPattern, value);
}

/**
 * 是否IPV4地址
 * @param value 要搜索匹配项的字符串
 */
export function isIpv4Address(value: string): boolean {
  const ipv4Pattern = pattern(RegexEnum.IPV4);
  return isMatch(ipv4Pattern, value);
}

/**
 * 是否IPV6地址
 * @param value 要搜索匹配项的字符串
 */
export function isIpv6Address(value: string): boolean {
  const ipv6Pattern = pattern(RegexEnum.IPV6);
  return isMatch(ipv6Pattern, value);
}

/**
 * 是否整数
 * @param value 要搜索匹配项的字符串
 */
export function isInteger(value: string): boolean {
  const integerPattern = pattern(RegexEnum.INTEGER);
  return isMatch(integerPattern, value);
}

/**
 * 是否Unicode字符串
 * @param value 要搜索匹配项的字符串
 */
export function isUnicode(value: string): boolean {
  const unicodePattern = pattern(RegexEnum.UNICODE);
  return isMatch(unicodePattern, value);
}

/**
 * 是否Url字符串
 * @param value 要搜索匹配项的字符串
 */
export function isUrl(value: string): boolean {
  const urlPattern = pattern(RegexEnum.URL);
  return isMatch(urlPattern, value);
}

/**
 * 是否手机号码
 * @param value 要搜索匹配项的字符串
 */
export function isMobileNumber(value: string): boolean {
  const mobileNumberPattern = pattern(RegexEnum.MOBILE_NUMBER);
  return isMatch(mobileNumberPattern, value);
}

/**
 * 是否身份证号
 * @param value 要搜索匹配项的字符串
 */
export function isIdCard(value: string): boolean {
  const idCardPattern = pattern(RegexEnum.ID_CARD);
  return isMatch(idCardPattern, value);
}

/**
 * 是否GUID
 * @param value 要搜索匹配项的字符串
 */
export function isGuid(value: string): boolean {
  const guidPattern = pattern(RegexEnum.GUID);
  return isMatch(guidPattern, value);
}
