import type { Router, RouteRecordRaw } from 'vue-router';

import { usePermissionStoreWithOut } from '/@/store/modules/permission';

import { PageEnum } from '/@/enums/pageEnum';
import { useIdentityStoreWithOut } from '/@/store/modules/identity';

import { PAGE_NOT_FOUND_ROUTE } from '/@/router/routes/basic';

import { RootRoute } from '/@/router/routes';

const CONFIRM_EMAIL_PATH = PageEnum.BASE_CONFIRM_EMAIL;
const RESET_PASSWORD_PATH = PageEnum.BASE_RESET_PASSWORD;
const OAUTH_CALLBACK_PATH = PageEnum.BASE_OAUTH_CALLBACK;
const LOGIN_PATH = PageEnum.BASE_LOGIN;

const ROOT_PATH = RootRoute.path;

/** 白名单 */
const whitePathList: PageEnum[] = [
  LOGIN_PATH,
  CONFIRM_EMAIL_PATH,
  RESET_PASSWORD_PATH,
  OAUTH_CALLBACK_PATH,
];

/**
 * 创建权限保护
 * @param router 路由实例
 */
export function createPermissionGuard(router: Router) {
  const useIdentityStore = useIdentityStoreWithOut();
  const permissionStore = usePermissionStoreWithOut();
  router.beforeEach(async (to, from, next) => {
    if (
      from.path === ROOT_PATH &&
      to.path === PageEnum.BASE_HOME &&
      useIdentityStore.getUserInfo.homePath &&
      useIdentityStore.getUserInfo.homePath !== PageEnum.BASE_HOME
    ) {
      next(useIdentityStore.getUserInfo.homePath);
      return;
    }

    const accessToken = useIdentityStore.getAccessToken;

    // 可直接录入白名单（Whitelist can be directly entered）
    if (whitePathList.includes(to.path as PageEnum)) {
      if (to.path === LOGIN_PATH && accessToken) {
        const isSessionTimeout = useIdentityStore.getSessionTimeout;
        try {
          await useIdentityStore.afterLoginAction();
          if (!isSessionTimeout) {
            next((to.query?.redirect as string) || '/');
            return;
          }
        } catch {}
      }
      next();
      return;
    }
    // 令牌不存在（token does not exist）
    if (!accessToken) {
      // 您可以不经许可访问。您需要将路由meta.ignoreAuth设置为true（You can access without permission. You need to set the routing meta.ignoreAuth to true）
      if (to.meta.ignoreAuth) {
        next();
        return;
      }

      // 重定向登录页（redirect login page）
      const redirectData: { path: string; replace: boolean; query?: Recordable<string> } = {
        path: LOGIN_PATH,
        replace: true,
      };
      if (to.path) {
        redirectData.query = {
          ...redirectData.query,
          redirect: to.path,
        };
      }
      next(redirectData);
      return;
    }

    // 处理完登录后跳转到404页面（Jump to the 404 page after processing the login）
    if (
      from.path === LOGIN_PATH &&
      to.name === PAGE_NOT_FOUND_ROUTE.name &&
      to.fullPath !== (useIdentityStore.getUserInfo.homePath || PageEnum.BASE_HOME)
    ) {
      next(useIdentityStore.getUserInfo.homePath || PageEnum.BASE_HOME);
      return;
    }

    // get userinfo while last fetch time is empty
    if (useIdentityStore.getLastUpdateTime === 0) {
      try {
        await useIdentityStore.getUserInfoAction();
      } catch (err) {
        next();
        return;
      }
    }

    if (permissionStore.getIsDynamicAddedRoute) {
      next();
      return;
    }

    const routes = await permissionStore.buildRoutesAction();

    routes.forEach((route) => {
      router.addRoute(route as unknown as RouteRecordRaw);
    });

    router.addRoute(PAGE_NOT_FOUND_ROUTE as unknown as RouteRecordRaw);

    permissionStore.setDynamicAddedRoute(true);

    if (to.name === PAGE_NOT_FOUND_ROUTE.name) {
      // 动态添加路由后，此处应当重定向到fullPath，否则会加载404页面内容
      next({ path: to.fullPath, replace: true, query: to.query });
    } else {
      const redirectPath = (from.query.redirect || to.path) as string;
      const redirect = decodeURIComponent(redirectPath);
      const nextData = to.path === redirect ? { ...to, replace: true } : { path: redirect };
      next(nextData);
    }
  });
}
